@@ -15,9 +15,8 @@ module Network.TLS.Extension
     , extensionID_ServerName
     , extensionID_MaxFragmentLength
     , extensionID_SecureRenegotiation
-    , extensionID_NextProtocolNegotiation
     , extensionID_ApplicationLayerProtocolNegotiation
-    , extensionID_EllipticCurves
+    , extensionID_NegotiatedGroups
     , extensionID_EcPointFormats
     , extensionID_Heartbeat
     , extensionID_SignatureAlgorithms
@@ -27,30 +26,22 @@ module Network.TLS.Extension
     , MaxFragmentLength(..)
     , MaxFragmentEnum(..)
     , SecureRenegotiation(..)
-    , NextProtocolNegotiation(..)
     , ApplicationLayerProtocolNegotiation(..)
-    , EllipticCurvesSupported(..)
-    , NamedCurve(..)
-    , CurveName(..)
+    , NegotiatedGroups(..)
+    , Group(..)
     , EcPointFormatsSupported(..)
     , EcPointFormat(..)
     , SessionTicket(..)
     , HeartBeat(..)
     , HeartBeatMode(..)
     , SignatureAlgorithms(..)
-    , availableEllipticCurves
     ) where
 
-import Control.Monad
-
-import Data.Word
-import Data.Maybe (fromMaybe, catMaybes)
-import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 
-import Network.TLS.Extension.EC
 import Network.TLS.Struct (ExtensionID, EnumSafe8(..), EnumSafe16(..), HashAndSignatureAlgorithm)
+import Network.TLS.Crypto.Types
 import Network.TLS.Wire
 import Network.TLS.Imports
 import Network.TLS.Packet (putSignatureHashAlgorithm, getSignatureHashAlgorithm)
@@ -68,7 +59,7 @@ extensionID_ServerName
   , extensionID_ClientAuthz
   , extensionID_ServerAuthz
   , extensionID_CertType
-  , extensionID_EllipticCurves
+  , extensionID_NegotiatedGroups
   , extensionID_EcPointFormats
   , extensionID_SRP
   , extensionID_SignatureAlgorithms
@@ -83,7 +74,6 @@ extensionID_ServerName
   , extensionID_EncryptThenMAC
   , extensionID_ExtendedMasterSecret
   , extensionID_SessionTicket
-  , extensionID_NextProtocolNegotiation
   , extensionID_SecureRenegotiation :: ExtensionID
 extensionID_ServerName                          = 0x0 -- RFC6066
 extensionID_MaxFragmentLength                   = 0x1 -- RFC6066
@@ -95,7 +85,7 @@ extensionID_UserMapping                         = 0x6 -- RFC4681
 extensionID_ClientAuthz                         = 0x7 -- RFC5878
 extensionID_ServerAuthz                         = 0x8 -- RFC5878
 extensionID_CertType                            = 0x9 -- RFC6091
-extensionID_EllipticCurves                      = 0xa -- RFC4492
+extensionID_NegotiatedGroups                    = 0xa -- RFC4492bis and TLS 1.3
 extensionID_EcPointFormats                      = 0xb -- RFC4492
 extensionID_SRP                                 = 0xc -- RFC5054
 extensionID_SignatureAlgorithms                 = 0xd -- RFC5246
@@ -110,7 +100,6 @@ extensionID_Padding                             = 0x15 -- draft-agl-tls-padding.
 extensionID_EncryptThenMAC                      = 0x16 -- RFC7366
 extensionID_ExtendedMasterSecret                = 0x17 -- draft-ietf-tls-session-hash. expires 2015-09-26
 extensionID_SessionTicket                       = 0x23 -- RFC4507
-extensionID_NextProtocolNegotiation             = 0x3374 -- obsolete
 extensionID_SecureRenegotiation                 = 0xff01 -- RFC5746
 
 definedExtensions :: [ExtensionID]
@@ -125,7 +114,7 @@ definedExtensions =
     , extensionID_ClientAuthz
     , extensionID_ServerAuthz
     , extensionID_CertType
-    , extensionID_EllipticCurves
+    , extensionID_NegotiatedGroups
     , extensionID_EcPointFormats
     , extensionID_SRP
     , extensionID_SignatureAlgorithms
@@ -140,7 +129,6 @@ definedExtensions =
     , extensionID_EncryptThenMAC
     , extensionID_ExtendedMasterSecret
     , extensionID_SessionTicket
-    , extensionID_NextProtocolNegotiation
     , extensionID_SecureRenegotiation
     ]
 
@@ -150,8 +138,7 @@ supportedExtensions = [ extensionID_ServerName
                       , extensionID_MaxFragmentLength
                       , extensionID_ApplicationLayerProtocolNegotiation
                       , extensionID_SecureRenegotiation
-                      , extensionID_NextProtocolNegotiation
-                      , extensionID_EllipticCurves
+                      , extensionID_NegotiatedGroups
                       , extensionID_EcPointFormats
                       , extensionID_SignatureAlgorithms
                       ]
@@ -165,8 +152,7 @@ class Extension a where
 -- | Server Name extension including the name type and the associated name.
 -- the associated name decoding is dependant of its name type.
 -- name type = 0 : hostname
-data ServerName = ServerName [ServerNameType]
-    deriving (Show,Eq)
+newtype ServerName = ServerName [ServerNameType] deriving (Show,Eq)
 
 data ServerNameType = ServerNameHostName HostName
                     | ServerNameOther    (Word8, ByteString)
@@ -177,7 +163,7 @@ instance Extension ServerName where
     extensionEncode (ServerName l) = runPut $ putOpaque16 (runPut $ mapM_ encodeNameType l)
         where encodeNameType (ServerNameHostName hn)       = putWord8 0  >> putOpaque16 (BC.pack hn) -- FIXME: should be puny code conversion
               encodeNameType (ServerNameOther (nt,opaque)) = putWord8 nt >> putBytes opaque
-    extensionDecode _ = runGetMaybe (getWord16 >>= \len -> getList (fromIntegral len) getServerName >>= return . ServerName)
+    extensionDecode _ = runGetMaybe (getWord16 >>= \len -> ServerName <$> getList (fromIntegral len) getServerName)
         where getServerName = do
                   ty    <- getWord8
                   sname <- getOpaque16
@@ -186,8 +172,7 @@ instance Extension ServerName where
                       _ -> ServerNameOther (ty, sname))
 
 -- | Max fragment extension with length from 512 bytes to 4096 bytes
-data MaxFragmentLength = MaxFragmentLength MaxFragmentEnum
-    deriving (Show,Eq)
+newtype MaxFragmentLength = MaxFragmentLength MaxFragmentEnum deriving (Show,Eq)
 data MaxFragmentEnum = MaxFragment512 | MaxFragment1024 | MaxFragment2048 | MaxFragment4096
     deriving (Show,Eq)
 
@@ -220,24 +205,8 @@ instance Extension SecureRenegotiation where
                  in return $ SecureRenegotiation cvd (Just svd)
            else return $ SecureRenegotiation opaque Nothing
 
--- | Next Protocol Negotiation
-data NextProtocolNegotiation = NextProtocolNegotiation [ByteString]
-    deriving (Show,Eq)
-
-instance Extension NextProtocolNegotiation where
-    extensionID _ = extensionID_NextProtocolNegotiation
-    extensionEncode (NextProtocolNegotiation bytes) =
-        runPut $ mapM_ putOpaque8 bytes
-    extensionDecode _ = runGetMaybe (NextProtocolNegotiation <$> getNPN)
-        where getNPN = do
-                 avail <- remaining
-                 case avail of
-                     0 -> return []
-                     _ -> do liftM2 (:) getOpaque8 getNPN
-
 -- | Application Layer Protocol Negotiation (ALPN)
-data ApplicationLayerProtocolNegotiation = ApplicationLayerProtocolNegotiation [ByteString]
-    deriving (Show,Eq)
+newtype ApplicationLayerProtocolNegotiation = ApplicationLayerProtocolNegotiation [ByteString] deriving (Show,Eq)
 
 instance Extension ApplicationLayerProtocolNegotiation where
     extensionID _ = extensionID_ApplicationLayerProtocolNegotiation
@@ -253,47 +222,16 @@ instance Extension ApplicationLayerProtocolNegotiation where
                      0 -> return []
                      _ -> (:) <$> getOpaque8 <*> getALPN'
 
-data EllipticCurvesSupported = EllipticCurvesSupported [NamedCurve]
-    deriving (Show,Eq)
 
-data NamedCurve =
-      SEC CurveName
-    | BrainPool BrainPoolCurve
-    | NamedCurve_arbitrary_explicit_prime_curves
-    | NamedCurve_arbitrary_explicit_char2_curves
-    deriving (Show,Eq)
-
-data BrainPoolCurve =
-      BrainPoolP512R1 -- 28
-    | BrainPoolP384R1 -- 27
-    | BrainPoolP256R1 -- 26
-    deriving (Show,Eq)
-
-availableEllipticCurves :: [NamedCurve]
-availableEllipticCurves = [SEC SEC_p256r1, SEC SEC_p384r1, SEC SEC_p521r1]
-
-instance EnumSafe16 NamedCurve where
-    fromEnumSafe16 NamedCurve_arbitrary_explicit_prime_curves = 0xFF01
-    fromEnumSafe16 NamedCurve_arbitrary_explicit_char2_curves = 0xFF02
-    fromEnumSafe16 (SEC nc) = maybe (error "named curve: internal error") id $ fromCurveName nc
-    fromEnumSafe16 (BrainPool BrainPoolP512R1) = 28
-    fromEnumSafe16 (BrainPool BrainPoolP384R1) = 27
-    fromEnumSafe16 (BrainPool BrainPoolP256R1) = 26
-    toEnumSafe16 0xFF01 = Just NamedCurve_arbitrary_explicit_prime_curves
-    toEnumSafe16 0xFF02 = Just NamedCurve_arbitrary_explicit_char2_curves
-    toEnumSafe16 26     = Just (BrainPool BrainPoolP256R1)
-    toEnumSafe16 27     = Just (BrainPool BrainPoolP384R1)
-    toEnumSafe16 28     = Just (BrainPool BrainPoolP512R1)
-    toEnumSafe16 n      = SEC <$> toCurveName n
+newtype NegotiatedGroups = NegotiatedGroups [Group] deriving (Show,Eq)
 
 -- on decode, filter all unknown curves
-instance Extension EllipticCurvesSupported where
-    extensionID _ = extensionID_EllipticCurves
-    extensionEncode (EllipticCurvesSupported curves) = runPut $ putWords16 $ map fromEnumSafe16 curves
-    extensionDecode _ = runGetMaybe (EllipticCurvesSupported . catMaybes . map toEnumSafe16 <$> getWords16)
+instance Extension NegotiatedGroups where
+    extensionID _ = extensionID_NegotiatedGroups
+    extensionEncode (NegotiatedGroups groups) = runPut $ putWords16 $ map fromEnumSafe16 groups
+    extensionDecode _ = runGetMaybe (NegotiatedGroups . mapMaybe toEnumSafe16 <$> getWords16)
 
-data EcPointFormatsSupported = EcPointFormatsSupported [EcPointFormat]
-    deriving (Show,Eq)
+newtype EcPointFormatsSupported = EcPointFormatsSupported [EcPointFormat] deriving (Show,Eq)
 
 data EcPointFormat =
       EcPointFormat_Uncompressed
@@ -315,18 +253,17 @@ instance EnumSafe8 EcPointFormat where
 instance Extension EcPointFormatsSupported where
     extensionID _ = extensionID_EcPointFormats
     extensionEncode (EcPointFormatsSupported formats) = runPut $ putWords8 $ map fromEnumSafe8 formats
-    extensionDecode _ = runGetMaybe (EcPointFormatsSupported . catMaybes . map toEnumSafe8 <$> getWords8)
+    extensionDecode _ = runGetMaybe (EcPointFormatsSupported . mapMaybe toEnumSafe8 <$> getWords8)
 
 data SessionTicket = SessionTicket
     deriving (Show,Eq)
 
 instance Extension SessionTicket where
     extensionID _ = extensionID_SessionTicket
-    extensionEncode (SessionTicket {}) = runPut $ return ()
+    extensionEncode SessionTicket{} = runPut $ return ()
     extensionDecode _ = runGetMaybe (return SessionTicket)
 
-data HeartBeat = HeartBeat HeartBeatMode
-    deriving (Show,Eq)
+newtype HeartBeat = HeartBeat HeartBeatMode deriving (Show,Eq)
 
 data HeartBeatMode =
       HeartBeat_PeerAllowedToSend
@@ -349,8 +286,7 @@ instance Extension HeartBeat where
             Just (Just mode) -> Just $ HeartBeat mode
             _                -> Nothing
 
-data SignatureAlgorithms = SignatureAlgorithms [HashAndSignatureAlgorithm]
-    deriving (Show,Eq)
+newtype SignatureAlgorithms = SignatureAlgorithms [HashAndSignatureAlgorithm] deriving (Show,Eq)
 
 instance Extension SignatureAlgorithms where
     extensionID _ = extensionID_SignatureAlgorithms
