@@ -56,6 +56,7 @@ import qualified Data.ByteString as B
 
 import Network.TLS.Types (Version(..))
 import Network.TLS.Cipher
+import Network.TLS.Imports
 import Data.Tuple (swap)
 
 import Crypto.Cipher.AES
@@ -123,7 +124,7 @@ noFail :: CryptoFailable a -> a
 noFail = throwCryptoError
 
 makeIV_ :: BlockCipher a => B.ByteString -> IV a
-makeIV_ = maybe (error "makeIV_") id . makeIV
+makeIV_ = fromMaybe (error "makeIV_") . makeIV
 
 tripledes_ede :: BulkDirection -> BulkKey -> BulkBlock
 tripledes_ede BulkEncrypt key =
@@ -134,7 +135,7 @@ tripledes_ede BulkDecrypt key =
      in (\iv input -> let output = cbcDecrypt ctx (tripledes_iv iv) input in (output, takelast 8 input))
 
 tripledes_iv :: BulkIV -> IV DES_EDE3
-tripledes_iv iv = maybe (error "tripledes cipher iv internal error") id $ makeIV iv
+tripledes_iv iv = fromMaybe (error "tripledes cipher iv internal error") $ makeIV iv
 
 rc4 :: BulkDirection -> BulkKey -> BulkStream
 rc4 _ bulkKey = BulkStream (combineRC4 $ RC4.initialize bulkKey)
@@ -174,6 +175,7 @@ ciphersuite_default =
     -- , cipher_RSA_3DES_EDE_CBC_SHA1
     ]
 
+{-# WARNING ciphersuite_all "This ciphersuite list contains RC4. Use ciphersuite_strong or ciphersuite_default instead." #-}
 -- | The default ciphersuites + some not recommended last resort ciphers.
 ciphersuite_all :: [Cipher]
 ciphersuite_all = ciphersuite_default ++
@@ -182,6 +184,7 @@ ciphersuite_all = ciphersuite_default ++
     , cipher_RC4_128_SHA1
     ]
 
+{-# DEPRECATED ciphersuite_medium "Use ciphersuite_strong or ciphersuite_default instead." #-}
 -- | list of medium ciphers.
 ciphersuite_medium :: [Cipher]
 ciphersuite_medium = [ cipher_RC4_128_SHA1
@@ -303,7 +306,7 @@ bulk_tripledes_ede = Bulk
 -- | unencrypted cipher using RSA for key exchange and MD5 for digest
 cipher_null_MD5 :: Cipher
 cipher_null_MD5 = Cipher
-    { cipherID           = 0x1
+    { cipherID           = 0x0001
     , cipherName         = "RSA-null-MD5"
     , cipherBulk         = bulk_null
     , cipherHash         = MD5
@@ -315,7 +318,7 @@ cipher_null_MD5 = Cipher
 -- | unencrypted cipher using RSA for key exchange and SHA1 for digest
 cipher_null_SHA1 :: Cipher
 cipher_null_SHA1 = Cipher
-    { cipherID           = 0x2
+    { cipherID           = 0x0002
     , cipherName         = "RSA-null-SHA1"
     , cipherBulk         = bulk_null
     , cipherHash         = SHA1
@@ -327,7 +330,7 @@ cipher_null_SHA1 = Cipher
 -- | RC4 cipher, RSA key exchange and MD5 for digest
 cipher_RC4_128_MD5 :: Cipher
 cipher_RC4_128_MD5 = Cipher
-    { cipherID           = 0x04
+    { cipherID           = 0x0004
     , cipherName         = "RSA-rc4-128-md5"
     , cipherBulk         = bulk_rc4
     , cipherHash         = MD5
@@ -339,7 +342,7 @@ cipher_RC4_128_MD5 = Cipher
 -- | RC4 cipher, RSA key exchange and SHA1 for digest
 cipher_RC4_128_SHA1 :: Cipher
 cipher_RC4_128_SHA1 = Cipher
-    { cipherID           = 0x05
+    { cipherID           = 0x0005
     , cipherName         = "RSA-rc4-128-sha1"
     , cipherBulk         = bulk_rc4
     , cipherHash         = SHA1
@@ -348,10 +351,22 @@ cipher_RC4_128_SHA1 = Cipher
     , cipherMinVer       = Nothing
     }
 
+-- | 3DES cipher (168 bit key), RSA key exchange and SHA1 for digest
+cipher_RSA_3DES_EDE_CBC_SHA1 :: Cipher
+cipher_RSA_3DES_EDE_CBC_SHA1 = Cipher
+    { cipherID           = 0x000A
+    , cipherName         = "RSA-3DES-EDE-CBC-SHA1"
+    , cipherBulk         = bulk_tripledes_ede
+    , cipherHash         = SHA1
+    , cipherPRFHash      = Nothing
+    , cipherKeyExchange  = CipherKeyExchange_RSA
+    , cipherMinVer       = Nothing
+    }
+
 -- | AES cipher (128 bit key), RSA key exchange and SHA1 for digest
 cipher_AES128_SHA1 :: Cipher
 cipher_AES128_SHA1 = Cipher
-    { cipherID           = 0x2f
+    { cipherID           = 0x002F
     , cipherName         = "RSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
@@ -363,7 +378,7 @@ cipher_AES128_SHA1 = Cipher
 -- | AES cipher (128 bit key), DHE key exchanged signed by DSA and SHA1 for digest
 cipher_DHE_DSS_AES128_SHA1 :: Cipher
 cipher_DHE_DSS_AES128_SHA1 = Cipher
-    { cipherID           = 0x32
+    { cipherID           = 0x0032
     , cipherName         = "DHE-DSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
@@ -375,7 +390,7 @@ cipher_DHE_DSS_AES128_SHA1 = Cipher
 -- | AES cipher (128 bit key), DHE key exchanged signed by RSA and SHA1 for digest
 cipher_DHE_RSA_AES128_SHA1 :: Cipher
 cipher_DHE_RSA_AES128_SHA1 = Cipher
-    { cipherID           = 0x33
+    { cipherID           = 0x0033
     , cipherName         = "DHE-RSA-AES128-SHA1"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
@@ -387,7 +402,7 @@ cipher_DHE_RSA_AES128_SHA1 = Cipher
 -- | AES cipher (256 bit key), RSA key exchange and SHA1 for digest
 cipher_AES256_SHA1 :: Cipher
 cipher_AES256_SHA1 = Cipher
-    { cipherID           = 0x35
+    { cipherID           = 0x0035
     , cipherName         = "RSA-AES256-SHA1"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA1
@@ -399,7 +414,7 @@ cipher_AES256_SHA1 = Cipher
 -- | AES cipher (256 bit key), DHE key exchanged signed by DSA and SHA1 for digest
 cipher_DHE_DSS_AES256_SHA1 :: Cipher
 cipher_DHE_DSS_AES256_SHA1 = cipher_DHE_DSS_AES128_SHA1
-    { cipherID           = 0x38
+    { cipherID           = 0x0038
     , cipherName         = "DHE-DSA-AES256-SHA1"
     , cipherBulk         = bulk_aes256
     }
@@ -407,7 +422,7 @@ cipher_DHE_DSS_AES256_SHA1 = cipher_DHE_DSS_AES128_SHA1
 -- | AES cipher (256 bit key), DHE key exchanged signed by RSA and SHA1 for digest
 cipher_DHE_RSA_AES256_SHA1 :: Cipher
 cipher_DHE_RSA_AES256_SHA1 = cipher_DHE_RSA_AES128_SHA1
-    { cipherID           = 0x39
+    { cipherID           = 0x0039
     , cipherName         = "DHE-RSA-AES256-SHA1"
     , cipherBulk         = bulk_aes256
     }
@@ -415,7 +430,7 @@ cipher_DHE_RSA_AES256_SHA1 = cipher_DHE_RSA_AES128_SHA1
 -- | AES cipher (128 bit key), RSA key exchange and SHA256 for digest
 cipher_AES128_SHA256 :: Cipher
 cipher_AES128_SHA256 = Cipher
-    { cipherID           = 0x3c
+    { cipherID           = 0x003C
     , cipherName         = "RSA-AES128-SHA256"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA256
@@ -427,7 +442,7 @@ cipher_AES128_SHA256 = Cipher
 -- | AES cipher (256 bit key), RSA key exchange and SHA256 for digest
 cipher_AES256_SHA256 :: Cipher
 cipher_AES256_SHA256 = Cipher
-    { cipherID           = 0x3d
+    { cipherID           = 0x003D
     , cipherName         = "RSA-AES256-SHA256"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA256
@@ -436,11 +451,36 @@ cipher_AES256_SHA256 = Cipher
     , cipherMinVer       = Just TLS12
     }
 
+-- This is not registered in IANA.
+-- So, this will be removed in the next major release.
+cipher_DHE_DSS_RC4_SHA1 :: Cipher
+cipher_DHE_DSS_RC4_SHA1 = cipher_DHE_DSS_AES128_SHA1
+    { cipherID           = 0x0066
+    , cipherName         = "DHE-DSA-RC4-SHA1"
+    , cipherBulk         = bulk_rc4
+    }
+
+cipher_DHE_RSA_AES128_SHA256 :: Cipher
+cipher_DHE_RSA_AES128_SHA256 = cipher_DHE_RSA_AES128_SHA1
+    { cipherID           = 0x0067
+    , cipherName         = "DHE-RSA-AES128-SHA256"
+    , cipherHash         = SHA256
+    , cipherPRFHash      = Just SHA256
+    , cipherMinVer       = Just TLS12
+    }
+
+cipher_DHE_RSA_AES256_SHA256 :: Cipher
+cipher_DHE_RSA_AES256_SHA256 = cipher_DHE_RSA_AES128_SHA256
+    { cipherID           = 0x006B
+    , cipherName         = "DHE-RSA-AES256-SHA256"
+    , cipherBulk         = bulk_aes256
+    }
+
 -- | AESGCM cipher (128 bit key), RSA key exchange.
 -- The SHA256 digest is used as a PRF, not as a MAC.
 cipher_AES128GCM_SHA256 :: Cipher
 cipher_AES128GCM_SHA256 = Cipher
-    { cipherID           = 0x9c
+    { cipherID           = 0x009C
     , cipherName         = "RSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
     , cipherHash         = SHA256
@@ -453,7 +493,7 @@ cipher_AES128GCM_SHA256 = Cipher
 -- The SHA384 digest is used as a PRF, not as a MAC.
 cipher_AES256GCM_SHA384 :: Cipher
 cipher_AES256GCM_SHA384 = Cipher
-    { cipherID           = 0x9d
+    { cipherID           = 0x009D
     , cipherName         = "RSA-AES256GCM-SHA384"
     , cipherBulk         = bulk_aes256gcm
     , cipherHash         = SHA384
@@ -462,44 +502,9 @@ cipher_AES256GCM_SHA384 = Cipher
     , cipherMinVer       = Just TLS12
     }
 
-cipher_DHE_DSS_RC4_SHA1 :: Cipher
-cipher_DHE_DSS_RC4_SHA1 = cipher_DHE_DSS_AES128_SHA1
-    { cipherID           = 0x66
-    , cipherName         = "DHE-DSA-RC4-SHA1"
-    , cipherBulk         = bulk_rc4
-    }
-
-cipher_DHE_RSA_AES128_SHA256 :: Cipher
-cipher_DHE_RSA_AES128_SHA256 = cipher_DHE_RSA_AES128_SHA1
-    { cipherID           = 0x67
-    , cipherName         = "DHE-RSA-AES128-SHA256"
-    , cipherHash         = SHA256
-    , cipherPRFHash      = Just SHA256
-    , cipherMinVer       = Just TLS12
-    }
-
-cipher_DHE_RSA_AES256_SHA256 :: Cipher
-cipher_DHE_RSA_AES256_SHA256 = cipher_DHE_RSA_AES128_SHA256
-    { cipherID           = 0x6b
-    , cipherName         = "DHE-RSA-AES256-SHA256"
-    , cipherBulk         = bulk_aes256
-    }
-
--- | 3DES cipher (168 bit key), RSA key exchange and SHA1 for digest
-cipher_RSA_3DES_EDE_CBC_SHA1 :: Cipher
-cipher_RSA_3DES_EDE_CBC_SHA1 = Cipher
-    { cipherID           = 0x0a
-    , cipherName         = "RSA-3DES-EDE-CBC-SHA1"
-    , cipherBulk         = bulk_tripledes_ede
-    , cipherHash         = SHA1
-    , cipherPRFHash      = Nothing
-    , cipherKeyExchange  = CipherKeyExchange_RSA
-    , cipherMinVer       = Nothing
-    }
-
 cipher_DHE_RSA_AES128GCM_SHA256 :: Cipher
 cipher_DHE_RSA_AES128GCM_SHA256 = Cipher
-    { cipherID           = 0x9e
+    { cipherID           = 0x009E
     , cipherName         = "DHE-RSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
     , cipherHash         = SHA256
@@ -510,7 +515,7 @@ cipher_DHE_RSA_AES128GCM_SHA256 = Cipher
 
 cipher_DHE_RSA_AES256GCM_SHA384 :: Cipher
 cipher_DHE_RSA_AES256GCM_SHA384 = Cipher
-    { cipherID           = 0x9f
+    { cipherID           = 0x009F
     , cipherName         = "DHE-RSA-AES256GCM-SHA384"
     , cipherBulk         = bulk_aes256gcm
     , cipherHash         = SHA384
@@ -521,7 +526,7 @@ cipher_DHE_RSA_AES256GCM_SHA384 = Cipher
 
 cipher_ECDHE_ECDSA_AES128CBC_SHA :: Cipher
 cipher_ECDHE_ECDSA_AES128CBC_SHA = Cipher
-    { cipherID           = 0xc009
+    { cipherID           = 0xC009
     , cipherName         = "ECDHE-ECDSA-AES128CBC-SHA"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
@@ -532,7 +537,7 @@ cipher_ECDHE_ECDSA_AES128CBC_SHA = Cipher
 
 cipher_ECDHE_ECDSA_AES256CBC_SHA :: Cipher
 cipher_ECDHE_ECDSA_AES256CBC_SHA = Cipher
-    { cipherID           = 0xc00A
+    { cipherID           = 0xC00A
     , cipherName         = "ECDHE-ECDSA-AES256CBC-SHA"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA1
@@ -543,7 +548,7 @@ cipher_ECDHE_ECDSA_AES256CBC_SHA = Cipher
 
 cipher_ECDHE_RSA_AES128CBC_SHA :: Cipher
 cipher_ECDHE_RSA_AES128CBC_SHA = Cipher
-    { cipherID           = 0xc013
+    { cipherID           = 0xC013
     , cipherName         = "ECDHE-RSA-AES128CBC-SHA"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA1
@@ -554,7 +559,7 @@ cipher_ECDHE_RSA_AES128CBC_SHA = Cipher
 
 cipher_ECDHE_RSA_AES256CBC_SHA :: Cipher
 cipher_ECDHE_RSA_AES256CBC_SHA = Cipher
-    { cipherID           = 0xc014
+    { cipherID           = 0xC014
     , cipherName         = "ECDHE-RSA-AES256CBC-SHA"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA1
@@ -565,7 +570,7 @@ cipher_ECDHE_RSA_AES256CBC_SHA = Cipher
 
 cipher_ECDHE_RSA_AES128CBC_SHA256 :: Cipher
 cipher_ECDHE_RSA_AES128CBC_SHA256 = Cipher
-    { cipherID           = 0xc027
+    { cipherID           = 0xC027
     , cipherName         = "ECDHE-RSA-AES128CBC-SHA256"
     , cipherBulk         = bulk_aes128
     , cipherHash         = SHA256
@@ -576,7 +581,7 @@ cipher_ECDHE_RSA_AES128CBC_SHA256 = Cipher
 
 cipher_ECDHE_RSA_AES256CBC_SHA384 :: Cipher
 cipher_ECDHE_RSA_AES256CBC_SHA384 = Cipher
-    { cipherID           = 0xc028
+    { cipherID           = 0xC028
     , cipherName         = "ECDHE-RSA-AES256CBC-SHA384"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA384
@@ -598,7 +603,7 @@ cipher_ECDHE_ECDSA_AES128CBC_SHA256 = Cipher
 
 cipher_ECDHE_ECDSA_AES256CBC_SHA384 :: Cipher
 cipher_ECDHE_ECDSA_AES256CBC_SHA384 = Cipher
-    { cipherID           = 0xc024
+    { cipherID           = 0xC024
     , cipherName         = "ECDHE-ECDSA-AES256CBC-SHA384"
     , cipherBulk         = bulk_aes256
     , cipherHash         = SHA384
@@ -609,7 +614,7 @@ cipher_ECDHE_ECDSA_AES256CBC_SHA384 = Cipher
 
 cipher_ECDHE_ECDSA_AES128GCM_SHA256 :: Cipher
 cipher_ECDHE_ECDSA_AES128GCM_SHA256 = Cipher
-    { cipherID           = 0xc02b
+    { cipherID           = 0xC02B
     , cipherName         = "ECDHE-ECDSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
     , cipherHash         = SHA256
@@ -620,7 +625,7 @@ cipher_ECDHE_ECDSA_AES128GCM_SHA256 = Cipher
 
 cipher_ECDHE_ECDSA_AES256GCM_SHA384 :: Cipher
 cipher_ECDHE_ECDSA_AES256GCM_SHA384 = Cipher
-    { cipherID           = 0xc02c
+    { cipherID           = 0xC02C
     , cipherName         = "ECDHE-ECDSA-AES256GCM-SHA384"
     , cipherBulk         = bulk_aes256gcm
     , cipherHash         = SHA384
@@ -631,7 +636,7 @@ cipher_ECDHE_ECDSA_AES256GCM_SHA384 = Cipher
 
 cipher_ECDHE_RSA_AES128GCM_SHA256 :: Cipher
 cipher_ECDHE_RSA_AES128GCM_SHA256 = Cipher
-    { cipherID           = 0xc02f
+    { cipherID           = 0xC02F
     , cipherName         = "ECDHE-RSA-AES128GCM-SHA256"
     , cipherBulk         = bulk_aes128gcm
     , cipherHash         = SHA256
@@ -642,7 +647,7 @@ cipher_ECDHE_RSA_AES128GCM_SHA256 = Cipher
 
 cipher_ECDHE_RSA_AES256GCM_SHA384 :: Cipher
 cipher_ECDHE_RSA_AES256GCM_SHA384 = Cipher
-    { cipherID           = 0xc030
+    { cipherID           = 0xC030
     , cipherName         = "ECDHE-RSA-AES256GCM-SHA384"
     , cipherBulk         = bulk_aes256gcm
     , cipherHash         = SHA384
@@ -651,94 +656,5 @@ cipher_ECDHE_RSA_AES256GCM_SHA384 = Cipher
     , cipherMinVer       = Just TLS12 -- RFC 5289
     }
 
-{-
-TLS 1.0 ciphers definition
-
-CipherSuite TLS_NULL_WITH_NULL_NULL               = { 0x00,0x00 };
-CipherSuite TLS_RSA_WITH_NULL_MD5                 = { 0x00,0x01 };
-CipherSuite TLS_RSA_WITH_NULL_SHA                 = { 0x00,0x02 };
-CipherSuite TLS_RSA_EXPORT_WITH_RC4_40_MD5        = { 0x00,0x03 };
-CipherSuite TLS_RSA_WITH_RC4_128_MD5              = { 0x00,0x04 };
-CipherSuite TLS_RSA_WITH_RC4_128_SHA              = { 0x00,0x05 };
-CipherSuite TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5    = { 0x00,0x06 };
-CipherSuite TLS_RSA_WITH_IDEA_CBC_SHA             = { 0x00,0x07 };
-CipherSuite TLS_RSA_EXPORT_WITH_DES40_CBC_SHA     = { 0x00,0x08 };
-CipherSuite TLS_RSA_WITH_DES_CBC_SHA              = { 0x00,0x09 };
-CipherSuite TLS_RSA_WITH_3DES_EDE_CBC_SHA         = { 0x00,0x0A };
-CipherSuite TLS_DH_DSS_EXPORT_WITH_DES40_CBC_SHA  = { 0x00,0x0B };
-CipherSuite TLS_DH_DSS_WITH_DES_CBC_SHA           = { 0x00,0x0C };
-CipherSuite TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA      = { 0x00,0x0D };
-CipherSuite TLS_DH_RSA_EXPORT_WITH_DES40_CBC_SHA  = { 0x00,0x0E };
-CipherSuite TLS_DH_RSA_WITH_DES_CBC_SHA           = { 0x00,0x0F };
-CipherSuite TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA      = { 0x00,0x10 };
-CipherSuite TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA = { 0x00,0x11 };
-CipherSuite TLS_DHE_DSS_WITH_DES_CBC_SHA          = { 0x00,0x12 };
-CipherSuite TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA     = { 0x00,0x13 };
-CipherSuite TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA = { 0x00,0x14 };
-CipherSuite TLS_DHE_RSA_WITH_DES_CBC_SHA          = { 0x00,0x15 };
-CipherSuite TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA     = { 0x00,0x16 };
-CipherSuite TLS_DH_anon_EXPORT_WITH_RC4_40_MD5    = { 0x00,0x17 };
-CipherSuite TLS_DH_anon_WITH_RC4_128_MD5          = { 0x00,0x18 };
-CipherSuite TLS_DH_anon_EXPORT_WITH_DES40_CBC_SHA = { 0x00,0x19 };
-CipherSuite TLS_DH_anon_WITH_DES_CBC_SHA          = { 0x00,0x1A };
-CipherSuite TLS_DH_anon_WITH_3DES_EDE_CBC_SHA     = { 0x00,0x1B };
-
-TLS-RSA-WITH-AES-128-GCM-SHA256    {0x00,0x9C}
-TLS-RSA-WITH-AES-256-GCM-SHA384    {0x00,0x9D}
-
-TLS-DHE-RSA-WITH-AES-128-CBC-SHA     {0x00,0x33}
-TLS-DHE-RSA-WITH-AES-256-CBC-SHA     {0x00,0x39}
-TLS-DHE-RSA-WITH-AES-128-CBC-SHA256   {0x00,0x67}
-TLS-DHE-RSA-WITH-AES-256-CBC-SHA256   {0x00,0x6B}
-TLS-DHE-RSA-WITH-AES-128-GCM-SHA256   {0x00,0x9E}
-TLS-DHE-RSA-WITH-AES-256-GCM-SHA384   {0x00,0x9F}
-TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA   {0x00,0x45}
-TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA   {0x00,0x88}
-TLS-DHE-RSA-WITH-CAMELLIA-128-CBC-SHA256      {0x00,0xBE}
-TLS-DHE-RSA-WITH-CAMELLIA-256-CBC-SHA256      {0x00,0xC4}
-TLS-DHE-RSA-WITH-CAMELLIA-128-GCM-SHA256      {0x00,0x7C}
-TLS-DHE-RSA-WITH-CAMELLIA-256-GCM-SHA256      {0x00,0x7D}
-TLS-DHE-RSA-WITH-3DES-EDE-CBC-SHA      {0x00,0x16}
-TLS-DHE-RSA-WITH-DES-CBC-SHA    {0x00,0x15}
-
-TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA     {0xC0,0x13}
-TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA     {0xC0,0x14}
-TLS-ECDHE-RSA-WITH-AES-128-CBC-SHA256   {0xC0,0x27}
-TLS-ECDHE-RSA-WITH-AES-256-CBC-SHA384   {0xC0,0x28}
-TLS-ECDHE-RSA-WITH-AES-128-GCM-SHA256   {0xC0,0x2F}
-TLS-ECDHE-RSA-WITH-AES-256-GCM-SHA384   {0xC0,0x30}
-TLS-ECDHE-RSA-WITH-CAMELLIA-128-CBC-SHA256    {0xC0,0x76}
-TLS-ECDHE-RSA-WITH-CAMELLIA-256-CBC-SHA384    {0xC0,0x77}
-TLS-ECDHE-RSA-WITH-CAMELLIA-128-GCM-SHA256    {0xC0,0x8A}
-TLS-ECDHE-RSA-WITH-CAMELLIA-256-GCM-SHA384    {0xC0,0x8B}
-TLS-ECDHE-RSA-WITH-3DES-EDE-CBC-SHA  {0xC0,0x12}
-TLS-ECDHE-RSA-WITH-RC4-128-SHA    {0xC0,0x11}
-TLS-ECDHE-RSA-WITH-NULL-SHA  {0xC0,0x10}
-
-TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA    {0xC0,0x09}
-TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA    {0xC0,0x0A}
-TLS-ECDHE-ECDSA-WITH-AES-128-CBC-SHA256 {0xC0,0x23}
-TLS-ECDHE-ECDSA-WITH-AES-256-CBC-SHA384 {0xC0,0x24}
-TLS-ECDHE-ECDSA-WITH-AES-128-GCM-SHA256 {0xC0,0x2B}
-TLS-ECDHE-ECDSA-WITH-AES-256-GCM-SHA384 {0xC0,0x2C}
-
-TLS-PSK-WITH-RC4-128-SHA    {0x00,0x8A}
-TLS-PSK-WITH-3DES-EDE-CBC-SHA      {0x00,0x8B}
-TLS-PSK-WITH-AES-128-CBC-SHA     {0x00,0x8C}
-TLS-PSK-WITH-AES-256-CBC-SHA     {0x00,0x8D}
-TLS-PSK-WITH-AES-128-CBC-SHA256   {0x00,0xAE}
-TLS-PSK-WITH-AES-256-CBC-SHA384   {0x00,0xAF}
-TLS-PSK-WITH-AES-128-GCM-SHA256   {0x00,0xA8}
-TLS-PSK-WITH-AES-256-GCM-SHA384   {0x00,0xA9}
-TLS-PSK-WITH-CAMELLIA-128-CBC-SHA256      {0xC0,0x94}
-TLS-PSK-WITH-CAMELLIA-256-CBC-SHA384      {0xC0,0x95}
-TLS-PSK-WITH-CAMELLIA-128-GCM-SHA256      {0xC0,0x8D}
-TLS-PSK-WITH-CAMELLIA-256-GCM-SHA384      {0xC0,0x8F}
-TLS-PSK-WITH-NULL-SHA     {0x00,0x2C}
-TLS-PSK-WITH-NULL-SHA256      {0x00,0xB4}
-TLS-PSK-WITH-NULL-SHA384      {0x00,0xB5}
-
-best ciphers suite description:
-    <http://www.thesprawl.org/research/tls-and-ssl-cipher-suites/>
-
--}
+-- A list of cipher suite is found from:
+-- https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-4
