@@ -1,5 +1,4 @@
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE CPP #-}
 -- |
 -- Module      : Network.TLS.Handshake.Server
 -- License     : BSD-style
@@ -30,19 +29,9 @@ import Network.TLS.Handshake.State
 import Network.TLS.Handshake.Process
 import Network.TLS.Handshake.Key
 import Network.TLS.Measurement
-import Data.Maybe (isJust, listToMaybe, mapMaybe)
-import Data.List (intersect)
 import qualified Data.ByteString as B
-import Data.ByteString.Char8 ()
-import Data.Ord (Down(..))
-#if MIN_VERSION_base(4,8,0)
-import Data.List (sortOn)
-#else
-import Data.List (sortBy)
-import Data.Ord (comparing)
-#endif
 
-import Control.Monad.State
+import Control.Monad.State.Strict
 
 import Network.TLS.Handshake.Signature
 import Network.TLS.Handshake.Common
@@ -82,7 +71,6 @@ handshakeServer sparams ctx = liftIO $ do
 --      <- client key xchg
 --      <- [cert verify]
 --      <- change cipher      -> change cipher
---      <- [NPN]
 --      <- finish             -> finish
 --      -> change cipher      <- change cipher
 --      -> finish             <- finish
@@ -141,48 +129,90 @@ handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientS
     -- such a cipher if no hash is available for it.  It's best to skip this
     -- cipher and pick another one (with another key exchange).
 
-    -- FIXME ciphers should also be checked for other requirements
-    -- (i.e. elliptic curves and D-H groups)
-    let cipherAllowed cipher = case chosenVersion of
-           TLS12 -> let -- Build a list of all signature algorithms with at least
-                        -- one hash algorithm in common between client and server.
-                        -- May contain duplicates, as it is only used for `elem`.
-                        possibleSigAlgs = map snd (hashAndSignaturesInCommon ctx exts)
+    -- Cipher selection is performed in two steps: first server credentials
+    -- are flagged as not suitable for signature if not compatible with
+    -- negotiated signature parameters.  Then ciphers are evalutated from
+    -- the resulting credentials.
 
-                        -- Check that a candidate cipher with a signature requiring
-                        -- a hash will have at least one hash available.  This avoids
-                        -- a failure later in 'decideHash'.
-                        hasSigningRequirements =
-                            case cipherKeyExchange cipher of
-                                CipherKeyExchange_DHE_RSA      -> SignatureRSA   `elem` possibleSigAlgs
-                                CipherKeyExchange_DHE_DSS      -> SignatureDSS   `elem` possibleSigAlgs
-                                CipherKeyExchange_ECDHE_RSA    -> SignatureRSA   `elem` possibleSigAlgs
-                                CipherKeyExchange_ECDHE_ECDSA  -> SignatureECDSA `elem` possibleSigAlgs
-                                _                              -> True -- signature not used
+    let possibleGroups   = negotiatedGroupsInCommon ctx exts
+        possibleECGroups = possibleGroups `intersect` availableECGroups
+        possibleFFGroups = possibleGroups `intersect` availableFFGroups
+        hasCommonGroupForECDHE = not (null possibleECGroups)
+        hasCommonGroupForFFDHE = not (null possibleFFGroups)
+        hasCustomGroupForFFDHE = isJust (serverDHEParams sparams)
+        canFFDHE = hasCustomGroupForFFDHE || hasCommonGroupForFFDHE
+        hasCommonGroup cipher =
+            case cipherKeyExchange cipher of
+                CipherKeyExchange_DH_Anon      -> canFFDHE
+                CipherKeyExchange_DHE_RSA      -> canFFDHE
+                CipherKeyExchange_DHE_DSS      -> canFFDHE
+                CipherKeyExchange_ECDHE_RSA    -> hasCommonGroupForECDHE
+                CipherKeyExchange_ECDHE_ECDSA  -> hasCommonGroupForECDHE
+                _                              -> True -- group not used
 
-                     in cipherAllowedForVersion chosenVersion cipher && hasSigningRequirements
-           _     -> cipherAllowedForVersion chosenVersion cipher
+        -- Ciphers are selected according to TLS version, availability of
+        -- (EC)DHE group and credential depending on key exchange.
+        cipherAllowed cipher   = cipherAllowedForVersion chosenVersion cipher && hasCommonGroup cipher
+        selectCipher credentials signatureCredentials = filter cipherAllowed (commonCiphers credentials signatureCredentials)
+
+        allCreds = extraCreds `mappend` sharedCredentials (ctxShared ctx)
+        (creds, signatureCreds, ciphersFilteredVersion)
+            = case chosenVersion of
+                  TLS12 -> let -- Build a list of all hash/signature algorithms in common between
+                               -- client and server.
+                               possibleHashSigAlgs = hashAndSignaturesInCommon ctx exts
+
+                               -- Check that a candidate signature credential will be compatible with
+                               -- client & server hash/signature algorithms.  This returns Just Int
+                               -- in order to sort credentials according to server hash/signature
+                               -- preference.  When the certificate has no matching hash/signature in
+                               -- 'possibleHashSigAlgs' the result is Nothing, and the credential will
+                               -- not be used to sign.  This avoids a failure later in 'decideHashSig'.
+                               signingRank cred =
+                                   case credentialDigitalSignatureAlg cred of
+                                       Just sig -> findIndex (sig `signatureCompatible`) possibleHashSigAlgs
+                                       Nothing  -> Nothing
+
+                               -- Finally compute credential lists and resulting cipher list.
+                               --
+                               -- We try to keep certificates supported by the client, but
+                               -- fallback to all credentials if this produces no suitable result
+                               -- (see RFC 5246 section 7.4.2 and TLS 1.3 section 4.4.2.2).
+                               -- The condition is based on resulting (EC)DHE ciphers so that
+                               -- filtering credentials does not give advantage to a less secure
+                               -- key exchange like CipherKeyExchange_RSA or CipherKeyExchange_DH_Anon.
+                               cltCreds    = filterCredentialsWithHashSignatures exts allCreds
+                               sigCltCreds = filterSortCredentials signingRank cltCreds
+                               sigAllCreds = filterSortCredentials signingRank allCreds
+                               cltCiphers  = selectCipher cltCreds sigCltCreds
+                               allCiphers  = selectCipher allCreds sigAllCreds
+
+                               resultTuple = if cipherListCredentialFallback cltCiphers
+                                                 then (allCreds, sigAllCreds, allCiphers)
+                                                 else (cltCreds, sigCltCreds, cltCiphers)
+                            in resultTuple
+                  _     -> (allCreds, allCreds, selectCipher allCreds allCreds)
 
     -- The shared cipherlist can become empty after filtering for compatible
     -- creds, check now before calling onCipherChoosing, which does not handle
     -- empty lists.
-    let ciphersFilteredVersion = filter cipherAllowed (commonCiphers extraCreds)
     when (null ciphersFilteredVersion) $ throwCore $
         Error_Protocol ("no cipher in common with the client", True, HandshakeFailure)
 
     let usedCipher = (onCipherChoosing $ serverHooks sparams) chosenVersion ciphersFilteredVersion
-        creds = extraCreds `mappend` sharedCredentials (ctxShared ctx)
 
     cred <- case cipherKeyExchange usedCipher of
-                CipherKeyExchange_RSA     -> return $ credentialsFindForDecrypting creds
-                CipherKeyExchange_DH_Anon -> return $ Nothing
-                CipherKeyExchange_DHE_RSA -> return $ credentialsFindForSigning SignatureRSA creds
-                CipherKeyExchange_DHE_DSS -> return $ credentialsFindForSigning SignatureDSS creds
-                CipherKeyExchange_ECDHE_RSA -> return $ credentialsFindForSigning SignatureRSA creds
-                _                         -> throwCore $ Error_Protocol ("key exchange algorithm not implemented", True, HandshakeFailure)
+                CipherKeyExchange_RSA       -> return $ credentialsFindForDecrypting creds
+                CipherKeyExchange_DH_Anon   -> return   Nothing
+                CipherKeyExchange_DHE_RSA   -> return $ credentialsFindForSigning RSA signatureCreds
+                CipherKeyExchange_DHE_DSS   -> return $ credentialsFindForSigning DSS signatureCreds
+                CipherKeyExchange_ECDHE_RSA -> return $ credentialsFindForSigning RSA signatureCreds
+                _                           -> throwCore $ Error_Protocol ("key exchange algorithm not implemented", True, HandshakeFailure)
 
     resumeSessionData <- case clientSession of
-            (Session (Just clientSessionId)) -> liftIO $ sessionResume (sharedSessionManager $ ctxShared ctx) clientSessionId
+            (Session (Just clientSessionId)) ->
+                let resume = liftIO $ sessionResume (sharedSessionManager $ ctxShared ctx) clientSessionId
+                 in validateSession serverName <$> resume
             (Session Nothing)                -> return Nothing
 
     maybe (return ()) (usingState_ ctx . setClientSNI) serverName
@@ -191,10 +221,6 @@ handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientS
         Just (ApplicationLayerProtocolNegotiation protos) -> usingState_ ctx $ setClientALPNSuggest protos
         _ -> return ()
 
-    case extensionLookup extensionID_EllipticCurves exts >>= extensionDecode False of
-        Just (EllipticCurvesSupported es) -> usingState_ ctx $ setClientEllipticCurveSuggest es
-        _ -> return ()
-
     -- Currently, we don't send back EcPointFormats. In this case,
     -- the client chooses EcPointFormat_Uncompressed.
     case extensionLookup extensionID_EcPointFormats exts >>= extensionDecode False of
@@ -204,11 +230,22 @@ handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientS
     doHandshake sparams cred ctx chosenVersion usedCipher usedCompression clientSession resumeSessionData exts
 
   where
-        commonCipherIDs extra = ciphers `intersect` map cipherID (ctxCiphers ctx extra)
-        commonCiphers   extra = filter (flip elem (commonCipherIDs extra) . cipherID) (ctxCiphers ctx extra)
+        commonCiphers creds sigCreds = filter ((`elem` ciphers) . cipherID) (getCiphers sparams creds sigCreds)
         commonCompressions    = compressionIntersectID (supportedCompressions $ ctxSupported ctx) compressions
         usedCompression       = head commonCompressions
 
+        validateSession _   Nothing                         = Nothing
+        validateSession sni m@(Just sd)
+            -- SessionData parameters are assumed to match the local server configuration
+            -- so we need to compare only to ClientHello inputs.  Abbreviated handshake
+            -- uses the same server_name than full handshake so the same
+            -- credentials (and thus ciphers) are available.
+            | clientVersion < sessionVersion sd             = Nothing
+            | sessionCipher sd `notElem` ciphers            = Nothing
+            | sessionCompression sd `notElem` compressions  = Nothing
+            | isJust sni && sessionClientSNI sd /= sni      = Nothing
+            | otherwise                                     = m
+
 
 handshakeServerWith _ _ _ = throwCore $ Error_Protocol ("unexpected handshake message received in handshakeServerWith", True, HandshakeFailure)
 
@@ -222,24 +259,19 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
             liftIO $ contextFlush ctx
             -- Receive client info until client Finished.
             recvClientData sparams ctx
-            sendChangeCipherAndFinish (return ()) ctx ServerRole
+            sendChangeCipherAndFinish ctx ServerRole
         Just sessionData -> do
             usingState_ ctx (setSession clientSession True)
             serverhello <- makeServerHello clientSession
             sendPacket ctx $ Handshake [serverhello]
             usingHState ctx $ setMasterSecret chosenVersion ServerRole $ sessionSecret sessionData
-            sendChangeCipherAndFinish (return ()) ctx ServerRole
+            sendChangeCipherAndFinish ctx ServerRole
             recvChangeCipherAndFinish ctx
     handshakeTerminate ctx
   where
-        clientRequestedNPN = isJust $ extensionLookup extensionID_NextProtocolNegotiation exts
         clientALPNSuggest = isJust $ extensionLookup extensionID_ApplicationLayerProtocolNegotiation exts
 
-        applicationProtocol = do
-            protos <- alpn
-            if null protos then npn else return protos
-
-        alpn | clientALPNSuggest = do
+        applicationProtocol | clientALPNSuggest = do
             suggest <- usingState_ ctx getClientALPNSuggest
             case (onALPNClientSuggest $ serverHooks sparams, suggest) of
                 (Just io, Just protos) -> do
@@ -251,20 +283,6 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
                                             (extensionEncode $ ApplicationLayerProtocolNegotiation [proto]) ]
                 (_, _)                  -> return []
              | otherwise = return []
-        npn = do
-            nextProtocols <-
-                if clientRequestedNPN
-                    then liftIO $ onSuggestNextProtocols $ serverHooks sparams
-                    else return Nothing
-            case nextProtocols of
-                Just protos -> do
-                    usingState_ ctx $ do
-                        setExtensionNPN True
-                        setServerNextProtocolSuggest protos
-                    return [ ExtensionRaw extensionID_NextProtocolNegotiation
-                             (extensionEncode $ NextProtocolNegotiation protos) ]
-                Nothing -> return []
-
 
         ---
         -- When the client sends a certificate, check whether
@@ -321,9 +339,9 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
             -- send server key exchange if needed
             skx <- case cipherKeyExchange usedCipher of
                         CipherKeyExchange_DH_Anon -> Just <$> generateSKX_DH_Anon
-                        CipherKeyExchange_DHE_RSA -> Just <$> generateSKX_DHE SignatureRSA
-                        CipherKeyExchange_DHE_DSS -> Just <$> generateSKX_DHE SignatureDSS
-                        CipherKeyExchange_ECDHE_RSA -> Just <$> generateSKX_ECDHE SignatureRSA
+                        CipherKeyExchange_DHE_RSA -> Just <$> generateSKX_DHE RSA
+                        CipherKeyExchange_DHE_DSS -> Just <$> generateSKX_DHE DSS
+                        CipherKeyExchange_ECDHE_RSA -> Just <$> generateSKX_ECDHE RSA
                         _                         -> return Nothing
             maybe (return ()) (sendPacket ctx . Handshake . (:[]) . ServerKeyXchg) skx
 
@@ -351,8 +369,17 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
         extractCAname cert = certSubjectDN $ getCertificate cert
 
         setup_DHE = do
-            let dhparams = fromJust "server DHE Params" $ serverDHEParams sparams
-            (priv, pub) <- generateDHE ctx dhparams
+            let possibleFFGroups = negotiatedGroupsInCommon ctx exts `intersect` availableFFGroups
+            (dhparams, priv, pub) <-
+                    case possibleFFGroups of
+                        []  ->
+                            let dhparams = fromJust "server DHE Params" $ serverDHEParams sparams
+                             in case findFiniteFieldGroup dhparams of
+                                    Just g  -> generateFFDHE ctx g
+                                    Nothing -> do
+                                        (priv, pub) <- generateDHE ctx dhparams
+                                        return (dhparams, priv, pub)
+                        g:_ -> generateFFDHE ctx g
 
             let serverParams = serverDHParamsFrom dhparams pub
 
@@ -366,50 +393,45 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
         -- the "signature_algorithms" extension in a client hello.
         -- If RSA is also used for key exchange, this function is
         -- not called.
-        decideHash sigAlg = do
+        decideHashSig sigAlg = do
             usedVersion <- usingState_ ctx getVersion
             case usedVersion of
               TLS12 -> do
                   let hashSigs = hashAndSignaturesInCommon ctx exts
-                  case filter ((==) sigAlg . snd) hashSigs of
+                  case filter (sigAlg `signatureCompatible`) hashSigs of
                       []  -> error ("no hash signature for " ++ show sigAlg)
-                      x:_ -> return $ Just (fst x)
+                      x:_ -> return $ Just x
               _     -> return Nothing
 
         generateSKX_DHE sigAlg = do
             serverParams  <- setup_DHE
-            mhash <- decideHash sigAlg
-            signed <- digitallySignDHParams ctx serverParams sigAlg mhash
+            mhashSig <- decideHashSig sigAlg
+            signed <- digitallySignDHParams ctx serverParams sigAlg mhashSig
             case sigAlg of
-                SignatureRSA -> return $ SKX_DHE_RSA serverParams signed
-                SignatureDSS -> return $ SKX_DHE_DSS serverParams signed
-                _            -> error ("generate skx_dhe unsupported signature type: " ++ show sigAlg)
+                RSA -> return $ SKX_DHE_RSA serverParams signed
+                DSS -> return $ SKX_DHE_DSS serverParams signed
+                _   -> error ("generate skx_dhe unsupported signature type: " ++ show sigAlg)
 
         generateSKX_DH_Anon = SKX_DH_Anon <$> setup_DHE
 
-        setup_ECDHE curvename = do
-            let ecdhparams = ecdhParams curvename
-            (priv, pub) <- generateECDHE ctx ecdhparams
-
-            let serverParams = ServerECDHParams ecdhparams pub
-
+        setup_ECDHE grp = do
+            (srvpri, srvpub) <- generateECDHE ctx grp
+            let serverParams = ServerECDHParams grp srvpub
             usingHState ctx $ setServerECDHParams serverParams
-            usingHState ctx $ setECDHPrivate priv
-            return (serverParams)
+            usingHState ctx $ setECDHPrivate srvpri
+            return serverParams
 
         generateSKX_ECDHE sigAlg = do
-            ncs <- usingState_ ctx getClientEllipticCurveSuggest
-            let common = availableEllipticCurves `intersect` fromJust "ClientEllipticCurveSuggest" ncs
-                -- FIXME: Currently maximum strength is chosen.
-                --        There may be a better way to choose EC.
-                nc = if null common then error "No common EllipticCurves"
-                                    else maximum $ map fromEnumSafe16 common
-            serverParams <- setup_ECDHE nc
-            mhash <- decideHash sigAlg
-            signed <- digitallySignECDHParams ctx serverParams sigAlg mhash
+            let possibleECGroups = negotiatedGroupsInCommon ctx exts `intersect` availableECGroups
+            grp <- case possibleECGroups of
+                     []  -> throwCore $ Error_Protocol ("no common group", True, HandshakeFailure)
+                     g:_ -> return g
+            serverParams <- setup_ECDHE grp
+            mhashSig <- decideHashSig sigAlg
+            signed <- digitallySignECDHParams ctx serverParams sigAlg mhashSig
             case sigAlg of
-                SignatureRSA -> return $ SKX_ECDHE_RSA serverParams signed
-                _            -> error ("generate skx_ecdhe unsupported signature type: " ++ show sigAlg)
+                RSA -> return $ SKX_ECDHE_RSA serverParams signed
+                _   -> error ("generate skx_ecdhe unsupported signature type: " ++ show sigAlg)
 
         -- create a DigitallySigned objects for DHParams or ECDHParams.
 
@@ -419,7 +441,6 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
 --      <- client key xchg
 --      <- [cert verify]
 --      <- change cipher
---      <- [NPN]
 --      <- finish
 --
 recvClientData :: ServerParams -> Context -> IO ()
@@ -468,33 +489,30 @@ recvClientData sparams ctx = runRecvState ctx (RecvStateHandshake processClientC
 
             -- FIXME should check certificate is allowed for signing
 
-            verif <- certificateVerifyCheck ctx usedVersion sigAlgExpected msgs dsig
+            verif <- checkCertificateVerify ctx usedVersion sigAlgExpected msgs dsig
 
-            case verif of
-                True -> do
-                    -- When verification succeeds, commit the
-                    -- client certificate chain to the context.
-                    --
-                    Just certs <- usingHState ctx getClientCertChain
-                    usingState_ ctx $ setClientCertificateChain certs
-                    return ()
-
-                False -> do
-                    -- Either verification failed because of an
-                    -- invalid format (with an error message), or
-                    -- the signature is wrong.  In either case,
-                    -- ask the application if it wants to
-                    -- proceed, we will do that.
-                    res <- liftIO $ onUnverifiedClientCert (serverHooks sparams)
-                    if res
-                        then do
-                            -- When verification fails, but the
-                            -- application callbacks accepts, we
-                            -- also commit the client certificate
-                            -- chain to the context.
-                            Just certs <- usingHState ctx getClientCertChain
-                            usingState_ ctx $ setClientCertificateChain certs
-                        else throwCore $ Error_Protocol ("verification failed", True, BadCertificate)
+            if verif then do
+                -- When verification succeeds, commit the
+                -- client certificate chain to the context.
+                --
+                Just certs <- usingHState ctx getClientCertChain
+                usingState_ ctx $ setClientCertificateChain certs
+                return ()
+              else do
+                -- Either verification failed because of an
+                -- invalid format (with an error message), or
+                -- the signature is wrong.  In either case,
+                -- ask the application if it wants to
+                -- proceed, we will do that.
+                res <- liftIO $ onUnverifiedClientCert (serverHooks sparams)
+                if res then do
+                        -- When verification fails, but the
+                        -- application callbacks accepts, we
+                        -- also commit the client certificate
+                        -- chain to the context.
+                        Just certs <- usingHState ctx getClientCertChain
+                        usingState_ ctx $ setClientCertificateChain certs
+                    else throwCore $ Error_Protocol ("verification failed", True, BadCertificate)
             return $ RecvStateNext expectChangeCipher
 
         processCertificateVerify p = do
@@ -508,18 +526,15 @@ recvClientData sparams ctx = runRecvState ctx (RecvStateHandshake processClientC
         getRemoteSignatureAlg = do
             pk <- usingHState ctx getRemotePublicKey
             case pk of
-                PubKeyRSA _   -> return SignatureRSA
-                PubKeyDSA _   -> return SignatureDSS
-                PubKeyEC  _   -> return SignatureECDSA
+                PubKeyRSA _   -> return RSA
+                PubKeyDSA _   -> return DSS
+                PubKeyEC  _   -> return ECDSA
                 _             -> throwCore $ Error_Protocol ("unsupported remote public key type", True, HandshakeFailure)
 
         expectChangeCipher ChangeCipherSpec = do
-            npn <- usingState_ ctx getExtensionNPN
-            return $ RecvStateHandshake $ if npn then expectNPN else expectFinish
-        expectChangeCipher p                = unexpected (show p) (Just "change cipher")
+            return $ RecvStateHandshake expectFinish
 
-        expectNPN (HsNextProtocolNegotiation _) = return $ RecvStateHandshake expectFinish
-        expectNPN p                             = unexpected (show p) (Just "Handshake NextProtocolNegotiation")
+        expectChangeCipher p                = unexpected (show p) (Just "change cipher")
 
         expectFinish (Finished _) = return RecvStateDone
         expectFinish p            = unexpected (show p) (Just "Handshake Finished")
@@ -547,14 +562,76 @@ hashAndSignaturesInCommon ctx exts =
         -- to server preference in 'supportedHashSignatures'.
      in sHashSigs `intersect` cHashSigs
 
+negotiatedGroupsInCommon :: Context -> [ExtensionRaw] -> [Group]
+negotiatedGroupsInCommon ctx exts = case extensionLookup extensionID_NegotiatedGroups exts >>= extensionDecode False of
+    Just (NegotiatedGroups clientGroups) ->
+        let serverGroups = supportedGroups (ctxSupported ctx)
+        in serverGroups `intersect` clientGroups
+    _                                    -> []
+
+credentialDigitalSignatureAlg :: Credential -> Maybe DigitalSignatureAlg
+credentialDigitalSignatureAlg cred =
+    findDigitalSignatureAlg (credentialPublicPrivateKeys cred)
+
+filterSortCredentials :: Ord a => (Credential -> Maybe a) -> Credentials -> Credentials
+filterSortCredentials rankFun (Credentials creds) =
+    let orderedPairs = sortOn fst [ (rankFun cred, cred) | cred <- creds ]
+     in Credentials [ cred | (Just _, cred) <- orderedPairs ]
+
+filterCredentialsWithHashSignatures :: [ExtensionRaw] -> Credentials -> Credentials
+filterCredentialsWithHashSignatures exts =
+    case extensionLookup extensionID_SignatureAlgorithms exts >>= extensionDecode False of
+        Nothing                        -> id
+        Just (SignatureAlgorithms sas) ->
+            let filterCredentials p (Credentials l) = Credentials (filter p l)
+             in filterCredentials (credentialMatchesHashSignatures sas)
+
+-- returns True if "signature_algorithms" certificate filtering produced no
+-- ephemeral D-H nor TLS13 cipher (so handshake with lower security)
+cipherListCredentialFallback :: [Cipher] -> Bool
+cipherListCredentialFallback xs = all nonDH xs
+  where
+    nonDH x = case cipherKeyExchange x of
+        CipherKeyExchange_DHE_RSA     -> False
+        CipherKeyExchange_DHE_DSS     -> False
+        CipherKeyExchange_ECDHE_RSA   -> False
+        CipherKeyExchange_ECDHE_ECDSA -> False
+        --CipherKeyExchange_TLS13       -> False
+        _                             -> True
+
 findHighestVersionFrom :: Version -> [Version] -> Maybe Version
 findHighestVersionFrom clientVersion allowedVersions =
     case filter (clientVersion >=) $ sortOn Down allowedVersions of
         []  -> Nothing
         v:_ -> Just v
 
-#if !MIN_VERSION_base(4,8,0)
-sortOn :: Ord b => (a -> b) -> [a] -> [a]
-sortOn f =
-  map snd . sortBy (comparing fst) . map (\x -> let y = f x in y `seq` (y, x))
-#endif
+-- We filter our allowed ciphers here according to dynamic credential lists.
+-- Credentials 'creds' come from server parameters but also SNI callback.
+-- When the key exchange requires a signature, we use a
+-- subset of this list named 'sigCreds'.  This list has been filtered in order
+-- to remove certificates that are not compatible with hash/signature
+-- restrictions (TLS 1.2).
+getCiphers :: ServerParams -> Credentials -> Credentials -> [Cipher]
+getCiphers sparams creds sigCreds = filter authorizedCKE (supportedCiphers $ serverSupported sparams)
+      where authorizedCKE cipher =
+                case cipherKeyExchange cipher of
+                    CipherKeyExchange_RSA         -> canEncryptRSA
+                    CipherKeyExchange_DH_Anon     -> True
+                    CipherKeyExchange_DHE_RSA     -> canSignRSA
+                    CipherKeyExchange_DHE_DSS     -> canSignDSS
+                    CipherKeyExchange_ECDHE_RSA   -> canSignRSA
+                    -- unimplemented: EC
+                    CipherKeyExchange_ECDHE_ECDSA -> False
+                    -- unimplemented: non ephemeral DH & ECDH.
+                    -- Note, these *should not* be implemented, and have
+                    -- (for example) been removed in OpenSSL 1.1.0
+                    --
+                    CipherKeyExchange_DH_DSS      -> False
+                    CipherKeyExchange_DH_RSA      -> False
+                    CipherKeyExchange_ECDH_ECDSA  -> False
+                    CipherKeyExchange_ECDH_RSA    -> False
+
+            canSignDSS    = DSS `elem` signingAlgs
+            canSignRSA    = RSA `elem` signingAlgs
+            canEncryptRSA = isJust $ credentialsFindForDecrypting creds
+            signingAlgs   = credentialsListSigningAlgorithms sigCreds
