@@ -14,7 +14,7 @@ module Network.TLS.Record.Disengage
         ( disengageRecord
         ) where
 
-import Control.Monad.State
+import Control.Monad.State.Strict
 
 import Crypto.Cipher.Types (AuthTag(..))
 import Network.TLS.Struct
@@ -28,7 +28,7 @@ import Network.TLS.Compression
 import Network.TLS.Util
 import Network.TLS.Wire
 import Network.TLS.Packet
-import Data.ByteString (ByteString)
+import Network.TLS.Imports
 import qualified Data.ByteString as B
 import qualified Data.ByteArray as B (convert)
 
@@ -62,14 +62,14 @@ getCipherData (Record pt ver _) cdata = do
         Just pad -> do
             cver <- getRecordVersion
             let b = B.length pad - 1
-            return (if cver < TLS10 then True else B.replicate (B.length pad) (fromIntegral b) `bytesEq` pad)
+            return (cver < TLS10 || B.replicate (B.length pad) (fromIntegral b) `bytesEq` pad)
 
     unless (macValid &&! paddingValid) $ do
         throwError $ Error_Protocol ("bad record mac", True, BadRecordMac)
 
     return $ cipherDataContent cdata
 
-decryptData :: Version -> Record Ciphertext -> Bytes -> RecordState -> RecordM Bytes
+decryptData :: Version -> Record Ciphertext -> ByteString -> RecordState -> RecordM ByteString
 decryptData ver record econtent tst = decryptOf (cstKey cst)
   where cipher     = fromJust "cipher" $ stCipher tst
         bulk       = cipherBulk cipher
@@ -82,12 +82,12 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
 
         sanityCheckError = throwError (Error_Packet "encrypted content too small for encryption parameters")
 
-        decryptOf :: BulkState -> RecordM Bytes
+        decryptOf :: BulkState -> RecordM ByteString
         decryptOf (BulkStateBlock decryptF) = do
             let minContent = (if explicitIV then bulkIVSize bulk else 0) + max (macSize + 1) blockSize
 
             -- check if we have enough bytes to cover the minimum for this cipher
-            when ((econtentLen `mod` blockSize) /= 0 || econtentLen < minContent) $ sanityCheckError
+            when ((econtentLen `mod` blockSize) /= 0 || econtentLen < minContent) sanityCheckError
 
             {- update IV -}
             (iv, econtent') <- if explicitIV
@@ -99,7 +99,7 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
             let paddinglength = fromIntegral (B.last content') + 1
             let contentlen = B.length content' - paddinglength - macSize
             (content, mac, padding) <- get3 content' (contentlen, macSize, paddinglength)
-            getCipherData record $ CipherData
+            getCipherData record CipherData
                     { cipherDataContent = content
                     , cipherDataMAC     = Just mac
                     , cipherDataPadding = Just padding
@@ -107,14 +107,14 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
 
         decryptOf (BulkStateStream (BulkStream decryptF)) = do
             -- check if we have enough bytes to cover the minimum for this cipher
-            when (econtentLen < macSize) $ sanityCheckError
+            when (econtentLen < macSize) sanityCheckError
 
             let (content', bulkStream') = decryptF econtent
             {- update Ctx -}
             let contentlen        = B.length content' - macSize
             (content, mac) <- get2 content' (contentlen, macSize)
             modify $ \txs -> txs { stCryptState = cst { cstKey = BulkStateStream bulkStream' } }
-            getCipherData record $ CipherData
+            getCipherData record CipherData
                     { cipherDataContent = content
                     , cipherDataMAC     = Just mac
                     , cipherDataPadding = Nothing
@@ -126,7 +126,7 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
                 cipherLen   = econtentLen - authTagLen - nonceExpLen
 
             -- check if we have enough bytes to cover the minimum for this cipher
-            when (econtentLen < (authTagLen + nonceExpLen)) $ sanityCheckError
+            when (econtentLen < (authTagLen + nonceExpLen)) sanityCheckError
 
             (enonce, econtent', authTag) <- get3 econtent (nonceExpLen, cipherLen, authTagLen)
             let encodedSeq = encodeWord64 $ msSequence $ stMacState tst
