@@ -13,8 +13,7 @@ module Network.TLS.Record.Engage
         ( engageRecord
         ) where
 
-import Control.Applicative
-import Control.Monad.State
+import Control.Monad.State.Strict
 import Crypto.Cipher.Types (AuthTag(..))
 
 import Network.TLS.Cap
@@ -24,7 +23,7 @@ import Network.TLS.Cipher
 import Network.TLS.Compression
 import Network.TLS.Wire
 import Network.TLS.Packet
-import Data.ByteString (ByteString)
+import Network.TLS.Imports
 import qualified Data.ByteString as B
 import qualified Data.ByteArray as B (convert)
 
