@@ -24,9 +24,7 @@ module Network.TLS.Record.State
     , getMacSequence
     ) where
 
-import Data.Word
-import Control.Applicative
-import Control.Monad.State
+import Control.Monad.State.Strict
 import Network.TLS.Compression
 import Network.TLS.Cipher
 import Network.TLS.ErrT
@@ -36,13 +34,14 @@ import Network.TLS.Wire
 import Network.TLS.Packet
 import Network.TLS.MAC
 import Network.TLS.Util
+import Network.TLS.Imports
 
 import qualified Data.ByteString as B
 
 data CryptState = CryptState
     { cstKey        :: !BulkState
-    , cstIV         :: !Bytes
-    , cstMacSecret  :: !Bytes
+    , cstIV         :: !ByteString
+    , cstMacSecret  :: !ByteString
     } deriving (Show)
 
 newtype MacState = MacState
@@ -106,7 +105,7 @@ incrRecordState :: RecordState -> RecordState
 incrRecordState ts = ts { stMacState = MacState (ms + 1) }
   where (MacState ms) = stMacState ts
 
-setRecordIV :: Bytes -> RecordState -> RecordState
+setRecordIV :: ByteString -> RecordState -> RecordState
 setRecordIV iv st = st { stCryptState = (stCryptState st) { cstIV = iv } }
 
 withCompression :: (Compression -> (Compression, a)) -> RecordM a
@@ -116,7 +115,7 @@ withCompression f = do
     put $ st { stCompression = nc }
     return a
 
-computeDigest :: Version -> RecordState -> Header -> Bytes -> (Bytes, RecordState)
+computeDigest :: Version -> RecordState -> Header -> ByteString -> (ByteString, RecordState)
 computeDigest ver tstate hdr content = (digest, incrRecordState tstate)
   where digest = macF (cstMacSecret cst) msg
         cst    = stCryptState tstate
@@ -128,7 +127,7 @@ computeDigest ver tstate hdr content = (digest, incrRecordState tstate)
             | ver < TLS10 = (macSSL hashA, B.concat [ encodedSeq, encodeHeaderNoVer hdr, content ])
             | otherwise   = (hmac hashA, B.concat [ encodedSeq, encodeHeader hdr, content ])
 
-makeDigest :: Header -> Bytes -> RecordM Bytes
+makeDigest :: Header -> ByteString -> RecordM ByteString
 makeDigest hdr content = do
     ver <- getRecordVersion
     st <- get
