@@ -16,8 +16,9 @@ module Network.TLS.Types
     , Direction(..)
     ) where
 
-import Data.ByteString (ByteString)
-import Data.Word
+import Network.TLS.Imports
+
+type HostName = String
 
 -- | Versions known to TLS
 --
@@ -29,9 +30,11 @@ type SessionID = ByteString
 
 -- | Session data to resume
 data SessionData = SessionData
-    { sessionVersion :: Version
-    , sessionCipher  :: CipherID
-    , sessionSecret  :: ByteString
+    { sessionVersion     :: Version
+    , sessionCipher      :: CipherID
+    , sessionCompression :: CompressionID
+    , sessionClientSNI   :: Maybe HostName
+    , sessionSecret      :: ByteString
     } deriving (Show,Eq)
 
 -- | Cipher identification
