@@ -1,115 +1,92 @@
+-- Disable this warning so we can still test deprecated functionality.
+{-# OPTIONS_GHC -fno-warn-warnings-deprecations #-}
 module Connection
     ( newPairContext
+    , arbitraryCiphers
+    , arbitraryVersions
+    , arbitraryHashSignatures
+    , arbitraryGroups
     , arbitraryPairParams
     , arbitraryPairParamsWithVersionsAndCiphers
     , arbitraryClientCredential
+    , isCustomDHParams
+    , leafPublicKey
+    , oneSessionManager
     , setPairParamsSessionManager
     , setPairParamsSessionResuming
     , establishDataPipe
     , initiateDataPipe
-    , blockCipher
-    , blockCipherDHE_RSA
-    , blockCipherDHE_DSS
-    , blockCipherECDHE_RSA
-    , blockCipherECDHE_RSA_SHA384
-    , streamCipher
     ) where
 
 import Test.Tasty.QuickCheck
 import Certificate
 import PubKey
 import PipeChan
-import Network.TLS
-import Network.TLS.Extra.FFDHE
+import Network.TLS as TLS
+import Network.TLS.Extra
 import Data.X509
 import Data.Default.Class
+import Data.IORef
 import Control.Applicative
 import Control.Concurrent.Chan
 import Control.Concurrent
 import qualified Control.Exception as E
+import Data.List (isInfixOf)
 
 import qualified Data.ByteString as B
 
+debug :: Bool
 debug = False
 
-blockCipher :: Cipher
-blockCipher = Cipher
-    { cipherID   = 0xff12
-    , cipherName = "rsa-id-const"
-    , cipherBulk = Bulk
-        { bulkName      = "id"
-        , bulkKeySize   = 16
-        , bulkIVSize    = 16
-        , bulkExplicitIV= 0
-        , bulkAuthTagLen= 0
-        , bulkBlockSize = 16
-        , bulkF         = BulkBlockF $ \_ _ _ -> (\m -> (m, B.empty))
-        }
-    , cipherHash        = MD5
-    , cipherPRFHash     = Nothing
-    , cipherKeyExchange = CipherKeyExchange_RSA
-    , cipherMinVer      = Nothing
-    }
-
-blockCipherDHE_RSA :: Cipher
-blockCipherDHE_RSA = blockCipher
-    { cipherID   = 0xff14
-    , cipherName = "dhe-rsa-id-const"
-    , cipherKeyExchange = CipherKeyExchange_DHE_RSA
-    }
-
-blockCipherDHE_DSS :: Cipher
-blockCipherDHE_DSS = blockCipher
-    { cipherID   = 0xff15
-    , cipherName = "dhe-dss-id-const"
-    , cipherKeyExchange = CipherKeyExchange_DHE_DSS
-    }
-
-blockCipherECDHE_RSA :: Cipher
-blockCipherECDHE_RSA = blockCipher
-    { cipherID   = 0xff16
-    , cipherName = "ecdhe-rsa-id-const"
-    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
-    }
-
-blockCipherECDHE_RSA_SHA384 :: Cipher
-blockCipherECDHE_RSA_SHA384 = blockCipher
-    { cipherID   = 0xff17
-    , cipherName = "ecdhe-rsa-id-const-sha384"
-    , cipherKeyExchange  = CipherKeyExchange_ECDHE_RSA
-    , cipherHash        = SHA384
-    , cipherPRFHash     = Just SHA384
-    , cipherMinVer      = Just TLS12
-    }
-
-streamCipher :: Cipher
-streamCipher = blockCipher
-    { cipherID   = 0xff13
-    , cipherBulk = Bulk
-        { bulkName      = "stream"
-        , bulkKeySize   = 16
-        , bulkIVSize    = 0
-        , bulkExplicitIV= 0
-        , bulkAuthTagLen= 0
-        , bulkBlockSize = 0
-        , bulkF         = BulkStreamF passThrough
-        }
-    }
-  where
-    passThrough _ _ = BulkStream go where go inp = (inp, BulkStream go)
-
 knownCiphers :: [Cipher]
-knownCiphers = [ blockCipher
-               , blockCipherDHE_RSA
-               , blockCipherDHE_DSS
-               , blockCipherECDHE_RSA
-               , blockCipherECDHE_RSA_SHA384
-               , streamCipher
-               ]
+knownCiphers = filter nonECDSA (ciphersuite_all ++ ciphersuite_weak)
+  where
+    ciphersuite_weak = [
+        cipher_DHE_DSS_RC4_SHA1
+      , cipher_RC4_128_MD5
+      , cipher_null_MD5
+      , cipher_null_SHA1
+      ]
+    -- arbitraryCredentialsOfEachType cannot generate ECDSA
+    nonECDSA c = not ("ECDSA" `isInfixOf` cipherName c)
+
+arbitraryCiphers :: Gen [Cipher]
+arbitraryCiphers = listOf1 $ elements knownCiphers
 
 knownVersions :: [Version]
 knownVersions = [SSL3,TLS10,TLS11,TLS12]
 
+arbitraryVersions :: Gen [Version]
+arbitraryVersions = sublistOf knownVersions
+
+knownHashSignatures :: [HashAndSignatureAlgorithm]
+knownHashSignatures = filter nonECDSA availableHashSignatures
+  where
+    availableHashSignatures = [(TLS.HashIntrinsic, SignatureRSApssSHA256)
+                              ,(TLS.HashSHA512, SignatureRSA)
+                              ,(TLS.HashSHA512, SignatureECDSA)
+                              ,(TLS.HashSHA384, SignatureRSA)
+                              ,(TLS.HashSHA384, SignatureECDSA)
+                              ,(TLS.HashSHA256, SignatureRSA)
+                              ,(TLS.HashSHA256, SignatureECDSA)
+                              ,(TLS.HashSHA1,   SignatureRSA)
+                              ,(TLS.HashSHA1,   SignatureDSS)
+                              ]
+    -- arbitraryCredentialsOfEachType cannot generate ECDSA
+    nonECDSA (_,s) = s /= SignatureECDSA
+
+arbitraryHashSignatures :: Gen [HashAndSignatureAlgorithm]
+arbitraryHashSignatures = sublistOf knownHashSignatures
+
+knownGroups, knownECGroups, knownFFGroups :: [Group]
+knownECGroups = [P256,P384,P521,X25519,X448]
+knownFFGroups = [FFDHE2048,FFDHE3072,FFDHE4096,FFDHE6144,FFDHE8192]
+knownGroups   = knownECGroups ++ knownFFGroups
+
+arbitraryGroups :: Gen [Group]
+arbitraryGroups = listOf1 $ elements knownGroups
+
+arbitraryCredentialsOfEachType :: Gen [(CertificateChain, PrivKey)]
 arbitraryCredentialsOfEachType = do
     let (pubKey, privKey) = getGlobalRSAPair
     (dsaPub, dsaPriv) <- arbitraryDSAPair
@@ -120,16 +97,21 @@ arbitraryCredentialsOfEachType = do
            , (PubKeyDSA dsaPub, PrivKeyDSA dsaPriv)
            ]
 
+isCustomDHParams :: DHParams -> Bool
+isCustomDHParams params = params == dhParams
+
+leafPublicKey :: CertificateChain -> Maybe PubKey
+leafPublicKey (CertificateChain [])       = Nothing
+leafPublicKey (CertificateChain (leaf:_)) = Just (certPubKey $ signedObject $ getSigned leaf)
+
 arbitraryCipherPair :: Version -> Gen ([Cipher], [Cipher])
 arbitraryCipherPair connectVersion = do
     serverCiphers      <- arbitraryCiphers `suchThat`
                                 (\cs -> or [maybe True (<= connectVersion) (cipherMinVer x) | x <- cs])
-    clientCiphers      <- oneof [arbitraryCiphers] `suchThat`
+    clientCiphers      <- arbitraryCiphers `suchThat`
                                 (\cs -> or [x `elem` serverCiphers &&
                                             maybe True (<= connectVersion) (cipherMinVer x) | x <- cs])
     return (clientCiphers, serverCiphers)
-  where
-        arbitraryCiphers  = resize (length knownCiphers + 1) $ listOf1 (elements knownCiphers)
 
 arbitraryPairParams :: Gen (ClientParams, ServerParams)
 arbitraryPairParams = do
@@ -142,6 +124,19 @@ arbitraryPairParams = do
     serAllowedVersions <- (:[]) `fmap` elements allowedVersions
     arbitraryPairParamsWithVersionsAndCiphers (allowedVersions, serAllowedVersions) (clientCiphers, serverCiphers)
 
+arbitraryECGroupPair :: Gen ([Group], [Group])
+arbitraryECGroupPair = do
+    let arbitraryECGroups = listOf1 $ elements knownECGroups
+    serverGroups <- arbitraryECGroups
+    clientGroups <- arbitraryECGroups `suchThat` any (`elem` serverGroups)
+    return (clientGroups, serverGroups)
+
+arbitraryHashSignaturePair :: Gen ([HashAndSignatureAlgorithm], [HashAndSignatureAlgorithm])
+arbitraryHashSignaturePair = do
+    serverHashSignatures <- shuffle knownHashSignatures
+    clientHashSignatures <- shuffle knownHashSignatures
+    return (clientHashSignatures, serverHashSignatures)
+
 arbitraryPairParamsWithVersionsAndCiphers :: ([Version], [Version])
                                           -> ([Cipher], [Cipher])
                                           -> Gen (ClientParams, ServerParams)
@@ -150,10 +145,14 @@ arbitraryPairParamsWithVersionsAndCiphers (clientVersions, serverVersions) (clie
     dhparams           <- elements [dhParams,ffdhe2048,ffdhe3072]
 
     creds              <- arbitraryCredentialsOfEachType
+    (clientGroups, serverGroups) <- arbitraryECGroupPair
+    (clientHashSignatures, serverHashSignatures) <- arbitraryHashSignaturePair
     let serverState = def
             { serverSupported = def { supportedCiphers  = serverCiphers
                                     , supportedVersions = serverVersions
                                     , supportedSecureRenegotiation = secNeg
+                                    , supportedGroups   = serverGroups
+                                    , supportedHashSignatures = serverHashSignatures
                                     }
             , serverDHEParams = Just dhparams
             , serverShared = def { sharedCredentials = Credentials creds }
@@ -162,6 +161,8 @@ arbitraryPairParamsWithVersionsAndCiphers (clientVersions, serverVersions) (clie
             { clientSupported = def { supportedCiphers  = clientCiphers
                                     , supportedVersions = clientVersions
                                     , supportedSecureRenegotiation = secNeg
+                                    , supportedGroups   = clientGroups
+                                    , supportedHashSignatures = clientHashSignatures
                                     }
             , clientShared = def { sharedValidationCache = ValidationCache
                                         { cacheAdd = \_ _ _ -> return ()
@@ -174,16 +175,31 @@ arbitraryPairParamsWithVersionsAndCiphers (clientVersions, serverVersions) (clie
 arbitraryClientCredential :: Gen Credential
 arbitraryClientCredential = arbitraryCredentialsOfEachType >>= elements
 
+-- | simple session manager to store one session id and session data for a single thread.
+-- a Real concurrent session manager would use an MVar and have multiples items.
+oneSessionManager :: IORef (Maybe (SessionID, SessionData)) -> SessionManager
+oneSessionManager ref = SessionManager
+    { sessionResume     = \myId     -> (>>= maybeResume myId) <$> readIORef ref
+    , sessionEstablish  = \myId dat -> writeIORef ref $ Just (myId, dat)
+    , sessionInvalidate = \_        -> return ()
+    }
+  where
+    maybeResume myId (sid, sdata)
+        | sid == myId = Just sdata
+        | otherwise   = Nothing
+
 setPairParamsSessionManager :: SessionManager -> (ClientParams, ServerParams) -> (ClientParams, ServerParams)
 setPairParamsSessionManager manager (clientState, serverState) = (nc,ns)
   where nc = clientState { clientShared = updateSessionManager $ clientShared clientState }
         ns = serverState { serverShared = updateSessionManager $ serverShared serverState }
         updateSessionManager shared = shared { sharedSessionManager = manager }
 
+setPairParamsSessionResuming :: (SessionID, SessionData) -> (ClientParams, ServerParams) -> (ClientParams, ServerParams)
 setPairParamsSessionResuming sessionStuff (clientState, serverState) =
     ( clientState { clientWantSessionResume = Just sessionStuff }
     , serverState)
 
+newPairContext :: PipeChan -> (ClientParams, ServerParams) -> IO (Context, Context)
 newPairContext pipe (cParams, sParams) = do
     let noFlush = return ()
     let noClose = return ()
@@ -204,6 +220,7 @@ newPairContext pipe (cParams, sParams) = do
                                     , loggingPacketRecv = putStrLn . ((pre ++ "<< ") ++) }
                 else def
 
+establishDataPipe :: (ClientParams, ServerParams) -> (Context -> Chan result -> IO ()) -> (Chan start -> Context -> IO ()) -> IO (Chan start, Chan result)
 establishDataPipe params tlsServer tlsClient = do
     -- initial setup
     pipe        <- newPipe
@@ -226,6 +243,7 @@ establishDataPipe params tlsServer tlsClient = do
                            ", supported: " ++ show supported
             E.throw e
 
+initiateDataPipe :: (ClientParams, ServerParams) -> (Context -> IO a1) -> (Context -> IO a) -> IO (Either E.SomeException a, Either E.SomeException a1)
 initiateDataPipe params tlsServer tlsClient = do
     -- initial setup
     pipe        <- newPipe
